#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#include "mypipe.h"

void Error(char *str, ...);

void 
hndl(int sig)
{
	char *p = "signal SIGINT received\n";

	write(1, p, strlen(p));
}

int
main(int argc, char **argv)
{
	int num = 1, stop = 0, fd, n, l;
	pid_t pid;
	char buf[IOBUFLEN];

	if (argc < 2) 
		Error("usage: %s r|w [ num [stop]]\n", argv[0]);
	
	if (argc >= 3)
		num = atoi(argv[2]);
	if (argc >= 4)
		stop = atoi(argv[3]);
	
	/* show system error if ^C is pressed */
	signal(SIGINT, hndl);

	if (!strcmp(argv[1], "w")) {
		printf("*** write %d times, stop = %d\n", num, stop);
		pid = getpid();

		fd = open(DEV_IN, O_WRONLY);
		if (fd < 0)
			Error("open write");
		for (n = 1; n <= num; ++n) {
			if (n == stop) {
				printf("stop> ");
				while (getchar() != '\n')
					;
			}

			sprintf(buf, "message %d from %d", n, pid);
			if (write(fd, buf, IOBUFLEN) != IOBUFLEN)
				Error("write");
			printf("written: %s\n", buf);
		}
	}
	else if (!strcmp(argv[1], "r")) {
		printf("*** read %d times, stop = %d\n", num, stop);

		fd = open(DEV_OUT, O_RDONLY);
		if (fd < 0)
			Error("open read");
		for (n = 1; n <= num; ++n) {
			if (n == stop) {
				printf("stop> ");
				while (getchar() != '\n')
					;
			}

			if ((l = read(fd, buf, IOBUFLEN)) < 0)
				Error("read");
			if (!l) {
				printf("EOF\n");
				break;
			}

			buf[l] = 0;
			printf("read: %s\n", buf);
		}
	}
	else 
		Error("unknown command");

	return 0;
}

void
Error(char *str, ...)
{
	int e;
        va_list ap;

	e = errno;

        va_start(ap, str);
        vfprintf(stderr, str, ap);
	va_end(ap);
        fprintf( stderr, "\n" );
	if (e)
		fprintf(stderr, "system error %d: %s\n", e, strerror(e));


        exit(1);
}

