/*
 * Link loop driver 5 times underneath mux driver. Wait until user presses CR.
 * Then unlink.
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "mux.h"
#include "loop.h"

void Error(char *str, ...);

#define N 5

struct loop_info {
	int	l_index;
};

struct loop_info loop_info[N];

int
main(int argc, char **argv)
{
	int mux_fd, fd, n, l, i;

	printf("using I_LINK, I_UNLINK\n");

	mux_fd = open(MUXDEV, O_RDWR);
	if (mux_fd < 0)
		Error("cannot open %s", MUXDEV);
		
	for (n = 0; n < N; ++n) {
		fd = open(LOOPDEV, O_RDWR);
		if (fd < 0)
			Error("cannot open %s", LOOPDEV);

#ifdef PUSH_GOUPPER		
		printf("pushing module \"goupper\"\n");
		if (ioctl(fd ,I_PUSH,"goupper") < 0)
			Error("ioctl(I_PUSH, \"goupper\")");
#endif
		
		l = ioctl(mux_fd, I_LINK, fd);
		if (l < 0)
			Error("ioctl I_LINK");
		printf("linked %s, %d, l_index %d\n", LOOPDEV, n, l);
		loop_info[n].l_index = l;

		if (close(fd) < 0)
			Error("close loop");
	}

	printf("streams linked. press CR to unlink> ");
	while (getchar() != '\n')
		;

	printf("unlinking\n");
	for (n = 0; n < N; ++n) {
		i = loop_info[n].l_index;
		l = ioctl(mux_fd, I_UNLINK, i);
		if (l < 0)
			Error("ioctl(I_UNLINK, %d)", i);
		printf("unlinked %s, %d\n", LOOPDEV, i);
	}
	printf("done\n");

	return 0;
}

void
Error(char *str, ...)
{
        va_list ap;
	int e;

	e = errno;

        va_start(ap, str);
        vfprintf(stderr, str, ap);
	va_end(ap);
        fprintf( stderr, "\n" );

	if (e)
		fprintf(stderr, "system error %d: %s\n", e, strerror(e));

        exit(1);
}
