/*
 * Print out all message types and pass them on. Uses strlog() calls. Use
 * strace command to view the trace output.
 */

#include <sys/types.h>
#include <sys/stream.h>
#include <sys/stropts.h>
#include <sys/conf.h>
#include <sys/errno.h>
#include <sys/cmn_err.h>
#include <sys/strlog.h>
#include <sys/log.h>

#include <sys/modctl.h>
#include <sys/kmem.h>
#include <sys/ddi.h>
#include <sys/sunddi.h>

static int spy_open(queue_t *q, dev_t *devp, int oflag, int sflag, 
	cred_t *credp);
static int spy_wput(queue_t *q, mblk_t *mp);
static int spy_wsrv(queue_t *q);
static int spy_rput(queue_t *q, mblk_t *mp);
static int spy_close(queue_t *q, int flag, cred_t *credp);

char *gettype(int t);

extern int nodev();

static struct module_info rminfo = {0x321, "spy", 0, INFPSZ, 0, 0};

static struct module_info wminfo = {0x321, "spy", 0, INFPSZ, STRLOW, STRHIGH};

static struct qinit rinit = {
	spy_rput, NULL, spy_open, spy_close, NULL, &rminfo, NULL,
};

static struct qinit winit = {
	spy_wput, NULL, NULL, NULL, NULL, &wminfo, NULL,
};

struct streamtab spy_info = {
	&rinit, &winit, NULL, NULL, 
};


static struct fmodsw spy_fsw = {
        "spy",
        &spy_info,
        D_NEW | D_MP | D_MTPERMOD
};

extern struct mod_ops mod_strmodops;

static struct modlstrmod md = {
        &mod_strmodops, "spy", &spy_fsw
};

static struct modlinkage ml = {
	MODREV_1,
	&md,
	NULL
};


/*
 * This is the module initialization routine.
 */

int
_init(void)
{
	return mod_install(&ml);
}

int
_info(struct modinfo *modinfop)
{
	return mod_info(&ml, modinfop);
}

int
_fini(void)
{
	return mod_remove(&ml);
}

static int
spy_open(queue_t *q, dev_t *devp, int oflag, int sflag, cred_t *credp)
{
	if (q->q_ptr)	/* already open */
		return 0;
	q->q_ptr = kmem_alloc(sizeof(char), KM_SLEEP); /* unused */
	if (q->q_ptr == NULL)
		return ENOMEM;
	WR(q)->q_ptr =q->q_ptr;
	qprocson(q);
	return 0;
}

static int
spy_close(queue_t *q, int oflag, cred_t *credp)
{
	qprocsoff(q);
	kmem_free(q->q_ptr, sizeof(char));
	return 0;
}

static int
spy_wput(queue_t *q, mblk_t *mp)
{
	struct iocblk	*ioc;
	struct copyreq	*cqp;
	struct copyresp	*csp;

	strlog(10, 20, 30, SL_TRACE, "*** spy_wput %s", 
		gettype(mp->b_datap->db_type));
	putnext(q, mp);

	return 0;
}

static int
spy_rput(queue_t *q, mblk_t *mp)
{
	strlog(10, 20, 30, SL_TRACE, "*** spy_rput %s", 
		gettype(mp->b_datap->db_type));
	putnext(q, mp);

	return 0;
}

char *
gettype(int t)
{
	switch (t) {
	    case M_DATA:	return "M_DATA";
	    case M_PROTO:	return "M_PROTO";
	    case M_BREAK:	return "M_BREAK";
	    case M_PASSFP:	return "M_PASSFP";
	    case M_EVENT:	return "M_EVENT";
	    case M_SIG:		return "M_SIG";
	    case M_DELAY:	return "M_DELAY";
	    case M_CTL:		return "M_CTL";
	    case M_IOCTL:	return "M_IOCTL";
	    case M_SETOPTS:	return "M_SETOPTS";
	    case M_RSE:		return "M_RSE";
	    case M_IOCACK:	return "M_IOCACK";
	    case M_IOCNAK:	return "M_IOCNAK";
	    case M_PCPROTO:	return "M_PCPROTO";
	    case M_PCSIG:	return "M_PCSIG";
	    case M_READ:	return "M_READ";
	    case M_FLUSH:	return "M_FLUSH";
	    case M_STOP:	return "M_STOP";
	    case M_START:	return "M_START";
	    case M_HANGUP:	return "M_HANGUP";
	    case M_ERROR:	return "M_ERROR";
	    case M_COPYIN:	return "M_COPYIN";
	    case M_COPYOUT:	return "M_COPYOUT";
	    case M_IOCDATA:	return "M_IOCDATA";
	    case M_PCRSE:	return "M_PCRSE";
	    case M_STOPI:	return "M_STOPI";
	    case M_STARTI:	return "M_STARTI";
	    case M_PCEVENT:	return "M_PCEVENT";
	    case M_UNHANGUP:	return "M_UNHANGUP";
	    default: return "???";
	}
}
