/*
 * usage: looptst [num [-n] ]
 *
 * Link /dev/loop under /dev/sspymux to test sspymux.
 *
 * Do num writes and reads in a loop to test the loopback device. In the end 
 * use a ioctl call to get the device status. With -n the read is
 * suppressed and if num is high enough, the output becomes blocked. In this 
 * case the ioctl is also blocked. The I_SETCLTIME and ioctl ic_timout are 
 * lowered to make that more bearable.
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "loop.h"
#include "sspy.h"

void
printstat(int fd)
{
	struct strioctl strioctl;
	struct loop_statistics stat;

	strioctl.ic_cmd = LOOP_I_GETSTAT;
	strioctl.ic_timout = 2;
	strioctl.ic_len = sizeof(stat);
	strioctl.ic_dp = (char *)&stat;

	printf("statistics:\n");

	if (ioctl(fd, I_STR, &strioctl) < 0) {
		perror("ioctl LOOP_I_GETSTAT");
		return;
	}

	printf("   num_msg %d, num_allocb %d, num_nomem %d\n",
		stat.num_msg, stat.num_allocb, stat.num_nomem);
	printf("   q_count %d, q_mblkcnt %d\n",
		stat.q_count, stat.q_mblkcnt);
}

main(int argc, char **argv)
{
	int fd, n, l, num;
	char s[80];
	int noread = 0;

	if (argc < 2)
		num = 10;
	else
		num = atoi(argv[1]);
	if (argc >= 3)
		noread = 1;
	printf("num %d, noread %d\n", num, noread);

	fd = open(SSPYMUXDEV, O_RDWR);
	if (fd < 0) {
		perror("open");
		exit(1);
	}

	/* close timeout in ms if write q is not empty, default = 15 sec */
	n = 100;
	if (ioctl(fd, I_SETCLTIME, &n) < 0) {
		perror("I_SETCLTIME");
		exit(1);
	}

	for (n = 0; n < num; ++n) {
		sprintf(s, "message %d", n);
		l = strlen(s);
		if (write(fd, s, l) != l) {
			perror("write");
			exit(1);
		}
		printf("written: %s", s);

		if (noread )
			printf("\n");
		else {
			l = read(fd, s, sizeof(s) - 1);
			if (l < 0) {
				perror("read");
				exit(1);
			}
			s[l] = 0;
			printf(",\tread: %s\n", s);
		}
	}

	printstat(fd);

	return 0;
}
