/*
 * Unlink device.
 *
 * usage: punlink l_index ...
 */

#include <sys/types.h>
#include <stropts.h>
#include <sys/conf.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <stdio.h>
#include <stdarg.h>
#include <string.h>

#include "sspy.h"

void Error(char *str, ...);

#define N 5

int
main(int argc, char **argv)
{
	int sspy_fd, fd, n, l, l_index;

	if (argc < 2)
		Error("usage: %s l_index ...", argv[0]);

	sspy_fd = open(SSPYDEV, O_RDWR);
	if (sspy_fd < 0)
		Error("cannot open %s", SSPYDEV);
		
	for (n = 1; n < argc; ++n) {
		l_index = atoi(argv[n]);
		printf("unlinking %d\n", l_index);

		l = ioctl(sspy_fd, I_PUNLINK, l_index);
		if (l < 0)
			Error("ioctl(I_UNLINK, %d)", l_index);
		printf("unlinked l_index %d\n", l_index);
	}

	return 0;
}

void
Error(char *str, ...)
{
        va_list ap;
	int e;

	e = errno;

        va_start(ap, str);
        vfprintf(stderr, str, ap);
	va_end(ap);
        fprintf( stderr, "\n" );

	if (e)
		fprintf(stderr, "system error %d: %s\n", e, strerror(e));

        exit(1);
}

