#define SSPYDEV		"/dev/sspy"
#define SSPYMUXDEV	"/dev/sspymux"

/* ioctls, only I_STR ioctls are supported */
#define SSPY_I_BASE (('s' << 24) | ('s' << 16) | ('p' << 8))

enum sspy_op {
  SSPY_OP_DRV_OPEN, SSPY_OP_MOD_OPEN, SSPY_OP_CLOSE, SSPY_OP_WPUT, SSPY_OP_RPUT
};

/* per instance dlpi bind info */
#define MAX_DLPI_ADDR 8
#define SSPY_DLPI_UNBOUND -1

struct dlpi_info {
	t_uscalar_t dl_sap;
	int addr_len;		/* SSPY_DLPI_UNBOUND if instance is unbound */
	uchar_t addr[MAX_DLPI_ADDR];
	int flags;		/* see below				    */
};

/* dlpi_info.flags */
#define SSPY_DLPI_DLIOCRAW	0x1	/* DLIOCRAW ioctl raw option set */


/* trace message control part */
struct sspy_trace_ctl {
	enum sspy_op	op;
	int		inst;	/* clone device instance	*/
	int		err;	/* os error for SSPY_OP_OPEN	*/
	unsigned	seqno;	/* sequence number		*/
	struct dlpi_info dlpi_info; /* dlpi bind info for inst.	*/
};

/* align address to next 64 bit address */
#define SSP_ALIGN8(p) ( (void *) (((unsigned long)(p) + 7UL) & ~7UL) )

/* Access macros for packed messages.
 *
 * Packed message format:
 *
 *    message type (M_*)	:	int, 4 byte
 *    message length		:	int, 4 byte
 *    message bytes 		:	"message length" bytes
 *    alignment to 64 bit border:	0 ... 7 bytes
 *
 * This structure is repeated for all b_cont parts of a message.
 * These macros can be applied to the data buffer returned by getmsg().
 * The buffer must be 64 bit alined for the macros to function correctly.
 */
#define SSP_MSG_TYPE(p)		( * (int *)(p) )
#define SSP_MSG_LEN(p)		( * ((int *)(p) + 1) )
#define SSP_MSG_START(p)	( (char *)(p) + 2 * sizeof(int) )
#define SSP_MSG_NEXT(p)		SSP_ALIGN8( SSP_MSG_START(p) + SSP_MSG_LEN(p) )
#define SSP_MSG_LEN_NEXT(p, l)	SSP_ALIGN8( SSP_MSG_START(p) + l )
